float4x4 World;
float4x4 View;
float4x4 Projection;

float4 AmbientColor;
float4 DiffuseColor;
float4 SpecularColor;
float SpecularPower;
float SpecularIntensity;
float3 LightDirection;
float3 EyePosition;

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : NORMAL0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
};

struct PixelShaderInput
{
	float4 Color : COLOR0;
};

// ambient only
VertexShaderOutput PerVertexAmbient(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	float ambientIntensity = 1.0f;
	output.Color = ambientIntensity * AmbientColor;

    return output;
}

// ambient and diffuse
VertexShaderOutput PerVertexDiffuse(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	float ambientIntensity = 1.0f;
	float4 ambientLightColor = ambientIntensity * AmbientColor;

	// transform a surface normal from model space to world space, and normalize it
	float3 worldNormal = normalize(mul(input.Normal, World));
	// The lambertian lighting calculation uses the direction from the vertex to the light
	float diffuseIntensity = saturate(dot(-LightDirection, worldNormal));
	float4 diffuseLightColor = diffuseIntensity * DiffuseColor;

	output.Color = diffuseLightColor + ambientLightColor;
	diffuseLightColor.a = 1.0;

    return output;
}

// ambient, diffuse, and specular
VertexShaderOutput PerVertexDiffusePhong(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);

	// transform a surface normal from model space to world space, and normalize it
	float3 worldNormal = normalize(mul(input.Normal, World));			
    // ensure the position coordinates are homogenous, we divide by W so that W == 1
	worldPosition = worldPosition/worldPosition.w;

	float ambientIntensity = 0.9f;
	float4 ambientLightColor = ambientIntensity * AmbientColor;

    // light direction 
    float3 directionToLight = normalize(LightDirection - worldPosition.xyz);
	// lambertian lighting
	float diffuseIntensity = saturate(dot(directionToLight, worldNormal));
	float4 diffuseLightColor = diffuseIntensity * DiffuseColor;

	// calculate a direct reflection vector for the light.
    float3 reflectionVector = normalize(reflect(-directionToLight, worldNormal));
	float3 directionToEye = normalize(EyePosition - worldPosition.xyz);
	float4 specularLightColor = SpecularIntensity * SpecularColor * pow(saturate(dot(reflectionVector, directionToEye)), SpecularPower);

	output.Color = specularLightColor + diffuseLightColor + ambientLightColor ;
	output.Color.a = 1.0;

    return output;
}

float4 SimplePixelShader(VertexShaderOutput input) : COLOR0
{
	return input.Color;
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 PerVertexAmbient();
        PixelShader = compile ps_2_0 SimplePixelShader();
    }
}

technique Technique2
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 PerVertexDiffuse();
        PixelShader = compile ps_2_0 SimplePixelShader();
    }
}

technique Technique3
{
    pass Pass1
    {
        VertexShader = compile vs_2_0 PerVertexDiffusePhong();
        PixelShader = compile ps_2_0 SimplePixelShader();
    }
}
